import { TypedBroadcastChannel } from '@genesislcap/foundation-broadcast-channel';
import { SetUserProperties, UserChannelEvents } from './types';
import type { User } from './user';
/**
 * @internal
 */
type WriteableUser = {
    -readonly [P in keyof User]: User[P];
};
/**
 * UserConfig DI interface.
 *
 * @public
 */
export interface UserConfig {
    /**
     * Exclude certain actions from tracking.
     *
     * See {@link UserPathHistory}
     */
    trackActionExclusions: unknown[];
    /**
     * Exclude certain paths from tracking.
     *
     * See {@link UserActionHistory}
     */
    trackPathExclusions: string[];
    /**
     * User property setter.
     *
     * @remarks
     * The user is typically set from the result of a LoginAck.
     */
    setUser(user: WriteableUser, properties: Partial<SetUserProperties>): User;
    /**
     * User property un-setter.
     */
    unsetUser(user: WriteableUser): User;
}
/**
 * Default UserConfig DI implementation.
 * @public
 */
export declare const defaultUserConfig: UserConfig;
/**
 * UserConfig DI key.
 *
 * @internal
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 */
export declare const UserConfig: import("@microsoft/fast-foundation").InterfaceSymbol<UserConfig>;
/**
 * UserChannel DI interface.
 * @example
 * ```ts
 * import { UserChannel } from '@genesislcap/foundation-user';
 * ...
 * @UserChannel protected channel: UserChannel;
 * ...
 * this.channel.onmessage = (e) => {
 *  if (this.channel.isMessageType('user-authentication-change', e)) {
 *    logger.debug('user-authentication-change', e.data.detail);
 *    return;
 *  }
 * }
 * ...
 * this.channel.close();
 * ```
 * @public
 */
export interface UserChannel extends TypedBroadcastChannel<UserChannelEvents> {
}
/**
 * UserChannel DI key.
 * @internal
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 */
export declare const UserChannel: import("@microsoft/fast-foundation").InterfaceSymbol<TypedBroadcastChannel<UserChannelEvents>>;
/**
 * Gets UserChannel from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getUserChannel } from '@genesislcap/foundation-user';
 * ...
 * const channel = getUserChannel();
 * ...
 * channel.onmessage = (e) => {
 *  if (channel.isMessageType('user-authentication-change', e)) {
 *    logger.debug('user-authentication-change', e.data.detail);
 *    return;
 *  }
 * }
 * ...
 * channel.close();
 * ```
 * @public
 */
export declare function getUserChannel(): UserChannel;
export {};
//# sourceMappingURL=config.d.ts.map