/**
 * UserAuthenticationChangeType.
 * @public
 */
export declare const UserAuthenticationChangeType = "user-authentication-change";
/**
 * UserAuthenticationChangeEvent.
 *
 * @remarks
 * Detail will either be `true` or `false` to indicate authentication status.
 *
 * @public
 */
export type UserAuthenticationChangeEvent = CustomEvent<boolean>;
/**
 * UserChannelId.
 * @public
 */
export declare const UserChannelId = "genesis-user";
/**
 * UserChannelEvents.
 * @public
 */
export type UserChannelEvents = {
    [UserAuthenticationChangeType]: boolean;
};
/**
 * UserPathHistory.
 *
 * @remarks
 * APIs to track a user's path history. Includes the pathname, search and hash only.
 * This is not intended proxy or replace the History API.
 *
 * @privateRemarks
 * Integrate with foundation analytics? Users could then simply call `user.trackPath` and we could engage analytics.
 * We should move analytics out of comms first.
 *
 * @public
 */
export interface UserPathHistory {
    /**
     * Array of paths that have been tracked.
     */
    readonly pathHistory: string[];
    /**
     * Track current path.
     */
    trackPath(): string;
    /**
     * Clears path history.
     */
    clearPaths(): void;
    /**
     * May be used to return to a target path after authentication.
     * @returns The last stored path.
     */
    lastPath(): string | undefined;
}
/**
 * UserActionHistory.
 *
 * @remarks
 * APIs to track a user's action history.
 *
 * @privateRemarks
 * Integrate with foundation analytics? Users could then simply call `user.trackAction` and we could engage analytics.
 * We should move analytics out of comms first.
 *
 * @public
 */
export interface UserActionHistory {
    /**
     * Array of actions that have been tracked.
     */
    readonly actionHistory: unknown[];
    /**
     * Track an action.
     */
    trackAction<TDetails = unknown>(details: TDetails): void;
    /**
     * Clears action history.
     */
    clearActions(): void;
    /**
     * @returns The last stored action.
     */
    lastAction(): unknown;
}
/**
 * @public
 */
export interface UserKVsProperty {
    readonly [key: string]: any;
}
/**
 * @public
 */
export interface UserDetailsSystemProperty {
    readonly date: string;
    readonly [key: string]: any;
}
/**
 * @public
 */
export interface UserDetailsProductProperty {
    readonly name: string;
    readonly version: string;
    readonly [key: string]: any;
}
/**
 * @public
 */
export interface UserQueueProperties {
    readonly updateQueue: string;
    readonly zeroMqProxyModeEnabled: boolean;
    readonly zeroMqProxyInboundPort: number;
    readonly zeroMqProxyOutboundPort: number;
}
/**
 * @public
 */
export interface UserDetailsProperty {
    readonly system: UserDetailsSystemProperty;
    readonly product: UserDetailsProductProperty[];
    readonly heartbeatIntervalSecs: number;
    readonly failedLoginAttempts: number;
    readonly rejectedLoginAttempts: number;
    readonly lastLoginDatetime: number;
    readonly daysToPasswordExpiry: number;
    readonly [key: string]: any;
}
/**
 * SetUserProperties is a partial match of `LoginAck` from `@genesislcap/foundation-auth`.
 *
 * @public
 */
export interface SetUserProperties extends Partial<UserQueueProperties> {
    readonly sourceRef: string;
    readonly userName: string;
    readonly userKVs: UserKVsProperty;
    readonly details: UserDetailsProperty;
    readonly permission: string[];
    readonly profile: string[];
    readonly [key: string]: any;
}
//# sourceMappingURL=types.d.ts.map