import { Listener } from '@genesislcap/foundation-utils';
import { UserConfig } from './config';
import type { SetUserProperties, UserActionHistory, UserAuthenticationChangeEvent, UserPathHistory } from './types';
/**
 * @remarks
 * For backwards compatibility, will be removed in a future major release, likely v15.
 *
 * @public
 */
export interface LegacyUserAPI {
    /**
     * @deprecated - Use `userName` instead.
     */
    readonly username: string;
    /**
     * @deprecated - Use `permission` instead.
     */
    readonly permissions: string[];
    /**
     * @deprecated - Use `profile` instead.
     */
    readonly profiles: string[];
    /**
     * @deprecated - Use `isAdmin` instead.
     */
    hasAdminProfile(): boolean;
}
/**
 * User interface.
 *
 * @remarks
 * The user singleton, which always exists in either an anonymous or authenticated mode.
 *
 * @example
 * ```ts
 * import { User } from '@genesislcap/foundation-user';
 * ...
 * @User private user: User
 * ```
 *
 * @public
 */
export interface User extends UserPathHistory, UserActionHistory, LegacyUserAPI {
    readonly isAuthenticated: boolean;
    readonly isAnonymous: boolean;
    readonly userKVs: {
        [key: string]: any;
    };
    readonly details: SetUserProperties['details'];
    readonly permission: string[];
    readonly profile: string[];
    readonly sourceRef: string;
    readonly userName: string;
    /**
     * A structuredClone of the raw unmanaged properties used to set the user.
     *
     * @remarks
     * This offers developers an escape hatch to the raw properties. If you feel there's a gap in the User interface based
     * on your application requirements, please raise a request for such properties to be moved onto the interface.
     */
    readonly rawProperties: Partial<SetUserProperties>;
    /**
     * @param properties - The UserProperties used to populate the User.
     */
    set(properties: Partial<SetUserProperties>): User;
    unset(): User;
    isAdmin(): boolean;
    isBanned(): boolean;
    /**
     * @param permission - A permission string or an array of permission strings to check.
     */
    hasPermission(permission: string | string[]): boolean;
    /**
     * @param profile - A profile string or an array of profile strings to check.
     */
    hasProfile(profile: string | string[]): boolean;
    /**
     * @example
     * ```ts
     * const unsubscribe = this.user.addAuthenticationChangeListener((event) => {
     *   console.log('User authenticated?', event.detail);
     * });
     * ```
     *
     * @param listener - A listener function.
     */
    addAuthenticationChangeListener(listener: Listener<UserAuthenticationChangeEvent>): () => void;
    clearTrackingHistory(): void;
}
/**
 * Default User implementation.
 * @public
 */
export declare class DefaultUser implements User {
    /** @internal */
    protected config: UserConfig;
    /** {@inheritDoc UserPathHistory.pathHistory} */
    pathHistory: string[];
    /** {@inheritDoc UserActionHistory.actionHistory} */
    actionHistory: unknown[];
    /** {@inheritDoc User.isAuthenticated} */
    isAuthenticated: boolean;
    /** {@inheritDoc User.userKVs} */
    userKVs: {
        [key: string]: any;
    };
    /** {@inheritDoc User.details} */
    details: SetUserProperties['details'];
    /** {@inheritDoc User.permission} */
    permission: string[];
    /** {@inheritDoc User.profile} */
    profile: string[];
    /** {@inheritDoc User.sourceRef} */
    sourceRef: string;
    /** {@inheritDoc User.userName} */
    userName: string;
    /** {@inheritDoc User.rawProperties} */
    rawProperties: Partial<SetUserProperties>;
    /** {@inheritDoc User.set} */
    set(properties: Partial<SetUserProperties>): User;
    /** {@inheritDoc User.unset} */
    unset(): User;
    /** {@inheritDoc User.isAnonymous} */
    get isAnonymous(): boolean;
    /** {@inheritDoc User.isAdmin} */
    isAdmin(): boolean;
    /** {@inheritDoc User.isBanned} */
    isBanned(): boolean;
    /** {@inheritDoc User.hasPermission} */
    hasPermission(permission: string | string[]): boolean;
    /** {@inheritDoc User.hasProfile} */
    hasProfile(profile: string | string[]): boolean;
    /** {@inheritDoc UserPathHistory.trackPath} */
    trackPath(): string;
    /** {@inheritDoc UserPathHistory.clearPaths} */
    clearPaths(): void;
    /** {@inheritDoc UserPathHistory.lastPath} */
    lastPath(): string;
    /** {@inheritDoc UserActionHistory.trackAction} */
    trackAction<TDetails = unknown>(details: TDetails): void;
    /** {@inheritDoc UserActionHistory.clearActions} */
    clearActions(): void;
    /** {@inheritDoc UserActionHistory.lastAction} */
    lastAction(): unknown;
    /** {@inheritDoc User.clearTrackingHistory} */
    clearTrackingHistory(): void;
    /** @internal */
    protected hasKey(keys: string[], key: string | string[]): boolean;
    /** @internal */
    protected authenticationChangeListeners: import("@genesislcap/foundation-utils").Observer<UserAuthenticationChangeEvent>;
    /** {@inheritDoc User.addAuthenticationChangeListener} */
    addAuthenticationChangeListener(listener: Listener<UserAuthenticationChangeEvent>): () => void;
    /** @internal */
    protected isAuthenticatedChanged(): void;
    /**
     * Bridge legacy user apis below
     */
    /** @internal */
    protected userNameChanged(): void;
    /** {@inheritDoc LegacyUserAPI.username} */
    get username(): string;
    /** @internal */
    protected permissionChanged(): void;
    /** {@inheritDoc LegacyUserAPI.permissions} */
    get permissions(): string[];
    /** @internal */
    protected profileChanged(): void;
    /** {@inheritDoc LegacyUserAPI.profiles} */
    get profiles(): string[];
    /** {@inheritDoc LegacyUserAPI.hasAdminProfile} */
    hasAdminProfile(): boolean;
}
/**
 * User DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export declare const User: import("@microsoft/fast-foundation").InterfaceSymbol<User>;
/**
 * Gets User from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getUser, User } from '@genesislcap/foundation-user';
 * ...
 * private user: User = getUser();
 * ```
 *
 * @public
 */
export declare function getUser(): User;
//# sourceMappingURL=user.d.ts.map