import { registerTypedBroadcastChannel, } from '@genesislcap/foundation-broadcast-channel';
import { DI } from '@microsoft/fast-foundation';
import { logger } from './logger';
import { UserChannelId } from './types';
/**
 * Default UserConfig DI implementation.
 * @public
 */
export const defaultUserConfig = {
    trackActionExclusions: [],
    trackPathExclusions: [''],
    setUser(user, properties) {
        if (user.isAuthenticated) {
            throw new Error(`User already set. If this is intentional, please call User.unset first.`);
        }
        logger.debug('setUser', properties);
        /**
         * Create a clone of the raw properties.
         */
        user.rawProperties = structuredClone(properties);
        /**
         * Set managed properties
         */
        user.userKVs = properties.userKVs;
        user.details = properties.details;
        user.permission = properties.permission;
        user.profile = properties.profile;
        user.sourceRef = properties.sourceRef;
        user.userName = properties.userName;
        user.isAuthenticated = true;
        return user;
    },
    unsetUser(user) {
        logger.debug('unsetUser');
        /**
         * Unset raw properties ref
         */
        user.rawProperties = undefined;
        /**
         * Unset managed properties
         */
        user.userKVs = undefined;
        user.details = undefined;
        user.permission = undefined;
        user.profile = undefined;
        user.sourceRef = undefined;
        user.userName = undefined;
        user.isAuthenticated = false;
        return user;
    },
};
/**
 * UserConfig DI key.
 *
 * @internal
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 */
export const UserConfig = DI.createInterface((x) => x.instance(defaultUserConfig));
/**
 * UserChannel DI key.
 * @internal
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 */
export const UserChannel = registerTypedBroadcastChannel(UserChannelId);
/**
 * Gets UserChannel from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getUserChannel } from '@genesislcap/foundation-user';
 * ...
 * const channel = getUserChannel();
 * ...
 * channel.onmessage = (e) => {
 *  if (channel.isMessageType('user-authentication-change', e)) {
 *    logger.debug('user-authentication-change', e.data.detail);
 *    return;
 *  }
 * }
 * ...
 * channel.close();
 * ```
 * @public
 */
export function getUserChannel() {
    return DI.getOrCreateDOMContainer().get(UserChannel);
}
