import { __decorate } from "tslib";
import { createObserver } from '@genesislcap/foundation-utils';
import { Observable, observable } from '@microsoft/fast-element';
import { DI } from '@microsoft/fast-foundation';
import { getUserChannel, UserConfig } from './config';
import { UserAuthenticationChangeType } from './types';
/**
 * Default User implementation.
 * @public
 */
export class DefaultUser {
    constructor() {
        /** {@inheritDoc UserPathHistory.pathHistory} */
        this.pathHistory = [];
        /** {@inheritDoc UserActionHistory.actionHistory} */
        this.actionHistory = [];
        /** {@inheritDoc User.isAuthenticated} */
        this.isAuthenticated = false;
        /** @internal */
        this.authenticationChangeListeners = createObserver();
    }
    /** {@inheritDoc User.set} */
    set(properties) {
        return this.config.setUser(this, properties);
    }
    /** {@inheritDoc User.unset} */
    unset() {
        return this.config.unsetUser(this);
    }
    /** {@inheritDoc User.isAnonymous} */
    get isAnonymous() {
        return !this.isAuthenticated;
    }
    /** {@inheritDoc User.isAdmin} */
    isAdmin() {
        return this.hasProfile('ADMIN');
    }
    /** {@inheritDoc User.isBanned} */
    isBanned() {
        return this.hasProfile('BANNED');
    }
    /** {@inheritDoc User.hasPermission} */
    hasPermission(permission) {
        return this.hasKey(this.permission, permission);
    }
    /** {@inheritDoc User.hasProfile} */
    hasProfile(profile) {
        return this.hasKey(this.profile, profile);
    }
    /** {@inheritDoc UserPathHistory.trackPath} */
    trackPath() {
        const { pathname, search, hash } = location;
        const path = `${pathname !== '/' ? pathname : ''}`;
        const urlPath = `${path}${search}${hash}`;
        if (this.config.trackPathExclusions.includes(urlPath)) {
            return;
        }
        this.pathHistory.push(urlPath);
        return urlPath;
    }
    /** {@inheritDoc UserPathHistory.clearPaths} */
    clearPaths() {
        this.pathHistory = [];
    }
    /** {@inheritDoc UserPathHistory.lastPath} */
    lastPath() {
        const length = this.pathHistory.length;
        return length > 0 ? this.pathHistory[length - 1] : undefined;
    }
    /** {@inheritDoc UserActionHistory.trackAction} */
    trackAction(details) {
        if (this.config.trackActionExclusions.includes(details)) {
            return;
        }
        this.actionHistory.push(details);
    }
    /** {@inheritDoc UserActionHistory.clearActions} */
    clearActions() {
        this.actionHistory = [];
    }
    /** {@inheritDoc UserActionHistory.lastAction} */
    lastAction() {
        const length = this.actionHistory.length;
        return length > 0 ? this.actionHistory[length - 1] : undefined;
    }
    /** {@inheritDoc User.clearTrackingHistory} */
    clearTrackingHistory() {
        this.clearPaths();
        this.clearActions();
    }
    /** @internal */
    hasKey(keys, key) {
        if (!keys || !keys.length || !key) {
            return false;
        }
        if (!Array.isArray(key)) {
            key = [key];
        }
        return key.some((k) => keys.includes(k));
    }
    /** {@inheritDoc User.addAuthenticationChangeListener} */
    addAuthenticationChangeListener(listener) {
        var _a;
        return (_a = this.authenticationChangeListeners) === null || _a === void 0 ? void 0 : _a.subscribe(listener);
    }
    /** @internal */
    isAuthenticatedChanged() {
        var _a;
        (_a = this.authenticationChangeListeners) === null || _a === void 0 ? void 0 : _a.publish(new CustomEvent(UserAuthenticationChangeType, {
            detail: this.isAuthenticated,
        }));
        /**
         * Normally we'd inject channels, but in objects like User that will always exist (no destroy), the use of a channel
         * may lead to tests failing to exit. Given authentication changes are infrequent, a temp channel poses less risk.
         */
        const channel = getUserChannel();
        channel.postMessage('user-authentication-change', this.isAuthenticated);
        channel.close();
    }
    /**
     * Bridge legacy user apis below
     */
    /** @internal */
    userNameChanged() {
        Observable.notify(this, 'username');
    }
    /** {@inheritDoc LegacyUserAPI.username} */
    get username() {
        Observable.track(this, 'username');
        return this.userName;
    }
    /** @internal */
    permissionChanged() {
        Observable.notify(this, 'permissions');
    }
    /** {@inheritDoc LegacyUserAPI.permissions} */
    get permissions() {
        Observable.track(this, 'permissions');
        return this.permission;
    }
    /** @internal */
    profileChanged() {
        Observable.notify(this, 'profiles');
    }
    /** {@inheritDoc LegacyUserAPI.profiles} */
    get profiles() {
        Observable.track(this, 'profiles');
        return this.profile;
    }
    /** {@inheritDoc LegacyUserAPI.hasAdminProfile} */
    hasAdminProfile() {
        return this.isAdmin();
    }
}
__decorate([
    UserConfig
], DefaultUser.prototype, "config", void 0);
__decorate([
    observable
], DefaultUser.prototype, "pathHistory", void 0);
__decorate([
    observable
], DefaultUser.prototype, "actionHistory", void 0);
__decorate([
    observable
], DefaultUser.prototype, "isAuthenticated", void 0);
__decorate([
    observable
], DefaultUser.prototype, "userKVs", void 0);
__decorate([
    observable
], DefaultUser.prototype, "details", void 0);
__decorate([
    observable
], DefaultUser.prototype, "permission", void 0);
__decorate([
    observable
], DefaultUser.prototype, "profile", void 0);
__decorate([
    observable
], DefaultUser.prototype, "sourceRef", void 0);
__decorate([
    observable
], DefaultUser.prototype, "userName", void 0);
__decorate([
    observable
], DefaultUser.prototype, "rawProperties", void 0);
/**
 * User DI key.
 *
 * @privateRemarks
 * Marked as internal to stop api-extractor becoming confused cross-linking tokens with the same name.
 *
 * @internal
 */
export const User = DI.createInterface((x) => x.singleton(DefaultUser));
/**
 * Gets User from the DI container.
 *
 * @remarks
 * A utility method for host applications that are not using decorators or the DI container.
 *
 * @example
 * ```ts
 * import { getUser, User } from '@genesislcap/foundation-user';
 * ...
 * private user: User = getUser();
 * ```
 *
 * @public
 */
export function getUser() {
    return DI.getOrCreateDOMContainer().get(User);
}
